const api_host = `${location.hostname}`
const api_port = 9000
const api_get_status = "api/v1/status"
const pathToDeviceStatus = `http://${api_host}:${api_port}/${api_get_status}`

init();


// Initialize application
function init() {
  console.log("Initializing...");

  fetchDeviceStatus(pathToDeviceStatus);
}

// fetchDeviceStatus - Gets device status json from AppO
function fetchDeviceStatus() {
  fetch(pathToDeviceStatus)
    .then(response => response.json())
    .then(data => {
      // Update device status widget with data from AppO
      updateDeviceStatusWidget(data)

      // Displays the app content after dynamically generating html elements
      document.getElementsByClassName('app-content')[0].classList.remove("hidden")
    })
    .catch(error => {
      console.error('Error:', error);
    });
}

// updateDeviceStatusWidget - Dynamically creates html elements onto page
function updateDeviceStatusWidget(device) {
  updateWidgetDeviceName(device.name)
  updateWidgetDeviceState(device.state)
  updateWidgetDeviceSystemInfo(device.info, device.firmware_unsupported)
  updateWidgetDeviceServices(device.services)
}

// updateWidgetDeviceName - Dynamically create device name element(s)
function updateWidgetDeviceName(name) {
  // Create a new p element
  var p = document.createElement("p");

  // Set attributes and content of the new p
  p.innerHTML = name

  // Append the new p to an existing element in the DOM
  var elements = document.getElementsByClassName("device-name");
  elements[0].appendChild(p);
}

// updateWidgetDeviceState - Dynamically create device status element(s)
function updateWidgetDeviceState(state) {
  // Create a new span element
  var span = document.createElement("span");

  // Switch on state
  // Currently missing Provisioning and Connected states
  switch (state) {
    case 0:
      span.innerHTML = "Error";
      span.classList.add("error")
    case 1:
      span.innerHTML = "Registered";
      break
    case 2:
      span.innerHTML = "Claimed";
      break
    case 3:
      span.innerHTML = "Provisioning";
      break
    case 4:
      span.innerHTML = "Connected";
      span.classList.add("success");
      break
  }

  // Search for div on device-status and append the span
  var elements = document.getElementsByClassName("device-status");
  elements[0].getElementsByTagName('div')[0].appendChild(span);
}

// updateWidgetDeviceSystemInfo - Dynamically create device system info element(s)
function updateWidgetDeviceSystemInfo(systemInfo, firmware_unsupported) {
  // Initialize loop for html elements
  let index = 0

  // Fetch all elements
  var elements = document.getElementsByClassName("system-info");

  // Loop through each info from the system info object
  for (let info in systemInfo) {
    // Create span
    var span = document.createElement("span");
    span.innerHTML = systemInfo[info];

    // Append on the current index
    elements[0].getElementsByTagName('div')[index].appendChild(span);

    // Increment to iterate to next div element
    index++;
  }

  // Create a new span element if the firmware is unsupported.
  if (firmware_unsupported) {
    var div = document.createElement("div")
    var span = document.createElement("span")
    span.classList.add("error")
    span.innerHTML = "Firmware Unsupported"
    var a = document.createElement("a")
    a.innerHTML = "More Details"
    a.href = "https://arc-cs-01.arcules.com/en/articles/8230726-onboarding-a-camera-to-cloud-device#h_e57e3a9e71"
    div.appendChild(span)
    div.appendChild(a)
    elements[0].appendChild(div);
  }
}

// updateWidgetDeviceServices - Dynamically create device services element(s)
function updateWidgetDeviceServices(services) {

  // in case the backend returns an empty array, null or just undefined
  if (typeof services === 'undefined' || services === null || !services.length) {
    return
  }

  // Get services element
  var elements = document.getElementsByClassName("services");

  // Iterate over each service
  for (let service of services) {
    // During each iteration, create the following elements:
    var div = document.createElement('div');

    var serviceNameSpan = document.createElement('span');
    var serviceStateSpan = document.createElement('span');
    // var serviceStatusSpan = document.createElement('span');

    // Set name and state
    serviceNameSpan.innerHTML = service.name;

    // state
    state = service.state.charAt(0).toUpperCase() + service.state.slice(1);
    serviceStateSpan.innerHTML = state;
    // serviceStatusSpan.innerHTML = service.status;

    // Append spans to div
    div.appendChild(serviceNameSpan);
    div.appendChild(serviceStateSpan);
    // div.appendChild(serviceStatusSpan);

    addTooltipToService(serviceStateSpan, service)

    // Append div to services
    elements[0].appendChild(div);
  }
}

function addTooltipToService(el, service) {
  tooltipStr = ""
  if (service.version != "") {
    tooltipStr += `version: ${service.version}`
  }

  if (service.status != "") {
    tooltipStr += ` status: ${service.status}`
  }

  // tooltip with service's version 
  el.setAttribute("tip", tooltipStr)
  let tip = document.createElement('span');
  tip.classList.add('tooltip');
  tip.innerText = el.getAttribute('tip');
  tip.style.transitionDelay = `0.2s`
  tip.style.transform = `translate(15px, 0)`
  el.appendChild(tip)
  el.onmousemove = (e) => {
    tip.style.left = el.clientX + 'px'
    tip.style.top = el.clientY + 'px';
  };
}